@@ -176,6 +176,12 @@ class RetryError(RuntimeError):
 @dataclass
 class AWSBatchConfig(AioAWSConfig):
 
+    #: defines a limit to the number of client connections
+    max_pool_connections: int = 100
+    # An asyncio.Semaphore to limit the number of concurrent clients;
+    # this defaults to the session client connection pool
+    sem: int = 100
+
     #: a batch job startup pause, ``random.uniform(start_pause, start_pause * 2)``;
     #: this applies when the job status is in ["SUBMITTED", "PENDING", "RUNNABLE"]
     start_pause: float = BATCH_STARTUP_PAUSE
@@ -183,24 +189,40 @@ class AWSBatchConfig(AioAWSConfig):
     #: an optional AioAWSBatchDB
     aio_batch_db: Optional[AioAWSBatchDB] = None
 
+    @property
+    def default_client_config(self) -> botocore.client.Config:
+        # botocore/config.py lists all the options
+        config = botocore.client.Config(
+            connect_timeout=30,
+            read_timeout=240,
+            max_pool_connections=self.max_pool_connections,
+        )
+        if self.aws_region:
+            config.region_name = self.aws_region
+        return config
+
     @asynccontextmanager
-    async def create_batch_client(self) -> aiobotocore.client.AioBaseClient:
+    async def create_batch_client(
+        self, *args, **kwargs
+    ) -> aiobotocore.client.AioBaseClient:
         """
         Create and yield an AWS Batch client using the ``AWSBatchConfig.session``
 
         :yield: an aiobotocore.client.AioBaseClient for AWS Batch
         """
-        async with self.session.create_client("batch") as client:
+        async with self.session.create_client("batch", *args, **kwargs) as client:
             yield client
 
     @asynccontextmanager
-    async def create_logs_client(self) -> aiobotocore.client.AioBaseClient:
+    async def create_logs_client(
+        self, *args, **kwargs
+    ) -> aiobotocore.client.AioBaseClient:
         """
         Create and yield an AWS CloudWatchLogs client using the ``AWSBatchConfig.session``
 
         :yield: an aiobotocore.client.AioBaseClient for AWS CloudWatchLogs
         """
-        async with self.session.create_client("logs") as client:
+        async with self.session.create_client("logs", *args, **kwargs) as client:
             yield client
 
 
@@ -364,7 +386,7 @@ async def aio_batch_update_jobs(jobs: Iterable[AWSBatchJob], config: AWSBatchCon
 
 
 async def aio_batch_job_logs(
-    job: AWSBatchJob, config: AWSBatchConfig = None
+    job: AWSBatchJob, config: AWSBatchConfig = None, skip_existing: bool = False
 ) -> Optional[List[Dict]]:
     """
     Asynchronous coroutine to get logs for a batch job log stream.  All
@@ -374,6 +396,8 @@ async def aio_batch_job_logs(
 
     :param job: A set of job parameters
     :param config: settings for task pauses between retries
+    :param skip_existing: skip jobs that already have logs; this is useful
+        if some jobs already have complete log events captured
     :return: a list of all the log events from get_log_events responses
     :raises: botocore.exceptions.ClientError
     :raises: RetryError if it exceeds retries
@@ -384,10 +408,18 @@ async def aio_batch_job_logs(
     if config is None:
         config = AWSBatchConfig.get_default_config()
 
+    if not job.job_id:
+        LOGGER.warning("AWS Batch job has no jobId, cannot fetch logs")
+        return
+
     if not job.job_description:
         LOGGER.warning("AWS Batch job has no description, cannot fetch logs")
         return
 
+    if skip_existing and job.logs:
+        LOGGER.warning("AWS Batch job has logs, skip fetch logs enabled")
+        return
+
     log_stream_name = job.job_description.get("container", {}).get("logStreamName")
     if not log_stream_name:
         LOGGER.warning(
@@ -418,13 +450,23 @@ async def aio_batch_job_logs(
                     await jitter(task_name, 0.0001, 0.01)
                     response = await logs_client.get_log_events(**kwargs)
 
-                    events = response.get("events", [])
-                    log_events.extend(events)
+                    log_page_events = response.get("events", [])
+                    log_events.extend(log_page_events)
 
-                    if forward_token != response["nextForwardToken"]:
-                        forward_token = response["nextForwardToken"]
-                    else:
+                    # # Note: for the record, cannot read the response as a stream
+                    # if response and response_success(response):
+                    #     response_events = response.get("events")
+                    #     if response_events:
+                    #         async with response_events as stream:
+                    #             log_page_events = await stream.read()
+                    #             log_events.extend(log_page_events)
+
+                    next_forward_token = response.get("nextForwardToken")
+                    if next_forward_token is None:
                         break
+                    if forward_token == next_forward_token:
+                        break
+                    forward_token = next_forward_token
 
                 if log_events:
                     LOGGER.info(
@@ -828,20 +870,27 @@ async def aio_batch_terminate_jobs(
             LOGGER.error(err)
 
 
-async def aio_batch_get_logs(jobs: Iterable[AWSBatchJob], config: AWSBatchConfig):
+async def aio_batch_get_logs(
+    jobs: Iterable[AWSBatchJob], config: AWSBatchConfig, skip_existing: bool = False
+):
     """
     Get job logs.  The logs should be updated in any
     configured jobs-db.
 
     :param jobs: any AWSBatchJob
     :param config: an AWSBatchConfig
+    :param skip_existing: skip jobs that already have logs; this is useful
+        if some jobs already have complete log events captured
     :return: each job maintains state, so this function returns nothing
     """
-    batch_tasks = [
-        asyncio.create_task(aio_batch_job_logs(job=job, config=config))
-        for job in jobs
-        if job.job_id
-    ]
+    batch_tasks = []
+    for job in jobs:
+        if job.job_id and job.job_description:
+            if skip_existing and job.logs:
+                continue
+            task = asyncio.create_task(aio_batch_job_logs(job=job, config=config))
+            batch_tasks.append(task)
+
     for task in asyncio.as_completed(batch_tasks):
         try:
             result = await task
@@ -850,6 +899,35 @@ async def aio_batch_get_logs(jobs: Iterable[AWSBatchJob], config: AWSBatchConfig
             LOGGER.error(err)
 
 
+def get_logs_by_status(
+    jobs: Iterable[AWSBatchJob],
+    job_states: List[str],
+    jobs_db: AioAWSBatchDB,
+    print_logs: bool = False,
+):
+    """
+    A utility function to get AWS Batch logs by job status, to
+    update the logs in a jobs-db, with an option to print the logs.
+
+    :param jobs: any :py:class:`AwsBatchJob` to filter by status
+    :param job_states: any AWS Batch job status
+    :param jobs_db: an instance of a :py:class:`AioAwsBatchDb`
+    :param print_logs: an option to print the logs
+    :return: None
+    """
+    jobs_by_status = list(
+        find_jobs_by_status(jobs=jobs, job_states=job_states, jobs_db=jobs_db)
+    )
+    if not jobs_by_status:
+        LOGGER.error("There are no jobs for: %s", job_states)
+        return
+    batch_get_logs(jobs=jobs_by_status, jobs_db=jobs_db)
+    if print_logs:
+        for job in jobs_by_status:
+            if job.logs:
+                print(job.logs)
+
+
 def job_for_status(job, job_states) -> Optional[AWSBatchJob]:
     if job.job_id and job.status in job_states:
         LOGGER.info(
@@ -1115,6 +1193,38 @@ def find_incomplete_jobs(
             yield db_job
 
 
+def find_latest_jobs_with_jobs_db(
+    jobs: Iterable[AWSBatchJob],
+    jobs_db: AioAWSBatchDB = None,
+) -> Generator[AWSBatchJob, None, None]:
+    """
+    Find the latest jobs data, using the job itself or a fallback to
+    check the latest data in the jobs-db.
+
+    This is most often used when jobs are regenerated for jobs that could
+    exist in the jobs-db; there are alternative methods on the jobs-db to
+    find all jobs matching various job states.
+
+    :param jobs: any AWSBatchJob
+    :param jobs_db: a jobs-db to check the latest
+        job status from the jobs-db, if it is not
+        available on the job (highly recommended)
+    :return: yield jobs found
+    """
+    LOGGER.info("Listing jobs")
+
+    for job in jobs:
+
+        # First check the job itself before checking the jobs_db
+        if job.job_id and job.status:
+            yield job
+
+        if jobs_db:
+            db_job = asyncio.run(jobs_db.find_latest_job_name(job.job_name))
+            if db_job:
+                yield db_job
+
+
 def batch_run_jobs(
     jobs: List[AWSBatchJob],
     jobs_db: AioAWSBatchDB = None,
@@ -1240,10 +1350,41 @@ def batch_update_jobs(
     asyncio.run(aio_batch_update_jobs(jobs=jobs, config=aio_batch_config))
 
 
+def batch_update_jobs_db(jobs_db: AioAWSBatchDB):
+    """
+    Update all the jobs-db job descriptions.  This is a synchronous wrapper
+    on :py:func:`aio_batch_update_jobs` for all jobs in a jobs-db.
+
+    :param jobs_db: a jobs-db to update
+    :return: the jobs-db is updated, so this function returns nothing
+    """
+    LOGGER.info("Updating jobs-db")
+
+    job_ids = list(asyncio.run(jobs_db.all_job_ids()))
+
+    aio_batch_config = AWSBatchConfig(
+        aio_batch_db=jobs_db,
+        min_pause=10,
+        max_pause=20,
+        start_pause=60,
+        max_pool_connections=1,
+        sem=500,
+    )
+    update_limit = 100  # AWS limit
+
+    for offset in range(0, len(job_ids), update_limit):
+        jobs_to_update = []
+        for job_id in job_ids[offset : offset + update_limit]:
+            job_doc = asyncio.run(jobs_db.find_by_job_id(job_id))
+            jobs_to_update.append(AWSBatchJob(**job_doc))
+        asyncio.run(aio_batch_update_jobs(jobs_to_update, aio_batch_config))
+
+
 def batch_get_logs(
     jobs: List[AWSBatchJob],
     jobs_db: AioAWSBatchDB = None,
     aio_batch_config: AWSBatchConfig = None,
+    skip_existing: bool = False,
 ):
     """
     Get job logs.
@@ -1253,6 +1394,8 @@ def batch_get_logs(
         this is only applied if an aio_batch_config is not provided
     :param aio_batch_config: a custom AWSBatchConfig; if provided,
         it is responsible for providing any optional jobs-db
+    :param skip_existing: skip jobs that already have logs; this is useful
+        if some jobs already have complete log events captured
     :return: each job maintains state, so this function returns nothing
     """
     # The polling is kept to a minimum to avoid interference with the batch API;
@@ -1264,15 +1407,19 @@ def batch_get_logs(
         # settings try to avoid rate throttling
         aio_batch_config = AWSBatchConfig(
             aio_batch_db=jobs_db,
-            min_jitter=3,
-            max_jitter=8,
-            min_pause=2,
+            min_jitter=5,
+            max_jitter=10,
+            min_pause=4,
             max_pause=10,
             start_pause=2,
-            max_pool_connections=1,
+            max_pool_connections=100,  # match semaphere (sem) value
             sem=100,
         )
-    asyncio.run(aio_batch_get_logs(jobs=jobs, config=aio_batch_config))
+    asyncio.run(
+        aio_batch_get_logs(
+            jobs=jobs, config=aio_batch_config, skip_existing=skip_existing
+        )
+    )
 
 
 def batch_cancel_jobs(
