@@ -25,13 +25,17 @@ Batch Job metadata models.
 
 import enum
 from dataclasses import dataclass
+from datetime import datetime
 from functools import total_ordering
+from math import floor
 from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Union
 
 from aio_aws.logger import get_logger
+from aio_aws.utils import datetime_from_unix_milliseconds
+from aio_aws.utils import http_date_to_timestamp
 
 LOGGER = get_logger(__name__)
 
@@ -126,25 +130,40 @@ class AWSBatchJob:
         type is optional, used only for job arrays
 
     :param container_overrides: a dictionary of container overrides.
-        Overrides include 'vcpus', 'memory', 'instanceType',
-        'environment', and 'resourceRequirements'. If the `command`
-        parameter is defined, it overrides the `container_overrides['command']`
+        Overrides include 'instanceType', 'environment', and 'resourceRequirements'.
 
-        The container_overrides might not override job definition ResourceRequirement
-        values. To override VCPU and MEMORY requirements that are in the
-        ResourceRequirement in the job definition, ResourceRequirement must be
-        specified in the SubmitJob request.
+        If the `command` parameter is defined, it overrides `container_overrides['command']`
 
-        vcpus (int): This parameter maps to CpuShares in the Create a container
+        To override VCPU and MEMORY requirements in the ResourceRequirement of the
+        job definition, a ResourceRequirement must be specified in the SubmitJob request.
+
+        https://docs.aws.amazon.com/batch/latest/APIReference/API_ResourceRequirement.html
+
+        VCPU (int): This parameter maps to CpuShares in the Create a container
         section of the Docker Remote API and the --cpu-shares option to docker run.
         Each vCPU is equivalent to 1,024 CPU shares. This parameter is supported for
         jobs that run on EC2 resources, but isn't supported for jobs that run on
         Fargate resources.
 
-        memory (int):  This parameter indicates the amount of memory (in MiB) that is
+        MEMORY (int):  This parameter indicates the amount of memory (in MiB) that is
         reserved for the job. This parameter is supported for jobs that run on EC2
         resources, but isn't supported for jobs that run on Fargate resources.
 
+        .. code-block::
+
+           container_overrides = {
+                "resourceRequirements": [
+                    {
+                        "type": "VCPU",
+                        "value": "2"
+                    },
+                    {
+                        "type": "MEMORY",
+                        "value": str(int(gib_to_mib(6)))
+                    }
+                ]
+            }
+
     :param max_tries: an optional limit to the number of job retries, which
         can apply in the job-manager function to any job with a SPOT failure
         only and it applies regardless of the job-definition settings.
@@ -237,13 +256,15 @@ class AWSBatchJob:
 
     def allow_submit_job(self) -> bool:
         """
-        - jobs with an existing job.job_id should skip
-        submission to avoid resubmission for the same job
+        Logic to determine whether a job can be submitted
+        or resubmitted:
 
+        - jobs with an existing job.job_id should skip
+          submission to avoid resubmission for the same job
         - jobs with too many tries cannot be resubmitted
 
         The :py:meth:`AWSBatchJob.reset()` can be applied
-        in order to resubmit any job that is no allowed.
+        in order to resubmit any job that is not allowed.
 
         :return: True if it is allowed
         """
@@ -267,23 +288,106 @@ class AWSBatchJob:
         self.status = None
         self.logs = None
 
+    @property
+    def submitted(self) -> Optional[int]:
+        """Timestamp (milliseconds since the epoch) for job submission"""
+        if self.job_submission:
+            try:
+                metadata = self.job_submission.get("ResponseMetadata", {})
+                date = metadata.get("HTTPHeaders", {}).get("date")
+                if date:
+                    # as an integer, it cannot round up into the future
+                    return floor(http_date_to_timestamp(date) * 1e3)
+            except Exception as err:
+                LOGGER.error(err)
+
+    @property
+    def submitted_datetime(self) -> Optional[datetime]:
+        """
+        Datetime for job submission;
+        this depends on setting the `job_submission`.
+        """
+        submitted_msec = self.submitted
+        if submitted_msec:
+            return datetime_from_unix_milliseconds(submitted_msec)
+
     @property
     def created(self) -> Optional[int]:
+        """
+        Timestamp (milliseconds since the epoch) for job createdAt;
+        this depends on updates to the jobDescription.
+
+        The Unix timestamp (in milliseconds) for when the job was created. For non-array jobs
+        and parent array jobs, this is when the job entered the SUBMITTED state (at the time
+        SubmitJob was called). For array child jobs, this is when the child job was spawned
+        by its parent and entered the PENDING state.
+        """
         if self.job_description:
             return self.job_description.get("createdAt")
 
+    @property
+    def created_datetime(self) -> Optional[datetime]:
+        """
+        Datetime for job createdAt;
+        this depends on updates to the jobDescription.
+        """
+        created_msec = self.created
+        if created_msec:
+            return datetime_from_unix_milliseconds(created_msec)
+
     @property
     def started(self) -> Optional[int]:
+        """
+        Timestamp (milliseconds since the epoch) for job startedAt;
+        this depends on updates to the jobDescription.
+
+        The Unix timestamp (in milliseconds) for when the job was started (when the job
+        transitioned from the STARTING state to the RUNNING state). This parameter isn't
+        provided for child jobs of array jobs or multi-node parallel jobs.
+        """
         if self.job_description:
             return self.job_description.get("startedAt")
 
+    @property
+    def started_datetime(self) -> Optional[datetime]:
+        """
+        Datetime for job startedAt;
+        this depends on updates to the jobDescription.
+        """
+        started_msec = self.started
+        if started_msec:
+            return datetime_from_unix_milliseconds(started_msec)
+
     @property
     def stopped(self) -> Optional[int]:
+        """
+        Timestamp (milliseconds since the epoch) for job stoppedAt;
+        this depends on updates to the jobDescription.
+
+        The Unix timestamp (in milliseconds) for when the job was stopped (when the job
+        transitioned from the RUNNING state to a terminal state, such as SUCCEEDED or FAILED).
+        """
         if self.job_description:
             return self.job_description.get("stoppedAt")
 
+    @property
+    def stopped_datetime(self) -> Optional[datetime]:
+        """
+        Datetime for job stoppedAt;
+        this depends on updates to the jobDescription.
+        """
+        stopped_msec = self.stopped
+        if stopped_msec:
+            return datetime_from_unix_milliseconds(stopped_msec)
+
     @property
     def elapsed(self) -> Optional[int]:
+        """
+        Timestamp (milliseconds since the epoch) for job elapsed time;
+        this depends on updates to the jobDescription;
+        it is the difference between createdAt and stoppedAt
+        (this can include long periods in a job queue).
+        """
         created = self.created
         stopped = self.stopped
         if stopped and created:
@@ -291,6 +395,11 @@ class AWSBatchJob:
 
     @property
     def runtime(self) -> Optional[int]:
+        """
+        Timestamp (milliseconds since the epoch) for job run time;
+        this depends on updates to the jobDescription;
+        it is the difference between startedAt and stoppedAt.
+        """
         started = self.started
         stopped = self.stopped
         if started and stopped:
@@ -298,6 +407,11 @@ class AWSBatchJob:
 
     @property
     def spinup(self) -> Optional[int]:
+        """
+        Timestamp (milliseconds since the epoch) for job startup time;
+        this depends on updates to the jobDescription;
+        it is the difference between createdAt and startedAt.
+        """
         created = self.created
         started = self.started
         if started and created:
