@@ -1,4 +1,4 @@
-# Copyright 2020 Darren Weber
+# Copyright 2019-2021 Darren Weber
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,8 +24,18 @@ import time
 logging.Formatter.converter = time.gmtime
 
 LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO").upper()
-LOG_FORMAT = "[%(levelname)s]  %(asctime)s.%(msecs)03dZ  %(name)s:%(funcName)s:%(lineno)d  %(message)s"
-LOG_FORMATTER = logging.Formatter(LOG_FORMAT, "%Y-%m-%dT%H:%M:%S")
+
+LOG_FORMAT = " | ".join(
+    [
+        "%(asctime)s.%(msecs)03dZ",
+        "%(levelname)s",
+        "%(name)s:%(funcName)s:%(lineno)d",
+        "%(message)s",
+    ]
+)
+LOG_DATEFMT = "%Y-%m-%dT%H:%M:%S"
+LOG_FORMATTER = logging.Formatter(LOG_FORMAT, LOG_DATEFMT)
+
 HANDLER = logging.StreamHandler(sys.stdout)
 HANDLER.formatter = LOG_FORMATTER
 
